#
# Project Atomic Unit Test Example Template
#
#pylint: skip-file
import unittest


class TestAtomicUnit(unittest.TestCase):
    """
    TODO: This test case ...
    """

    def test_unit_desired_behavior(self):
        """
        TODO: Verifies ...
        """
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()
